#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Tivion
## Copyright (C) 2009 Ángel Guzmán Maeso
## http://shakaran.es
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Tivion = TiViOn = TV Online
import constants as cons
print cons.VERSION

# [TODO]
# -Choose at start if there are some channel for reproduce directly
# -Screenshot folder should be customizable by interface

import sys
import os
from subprocess import Popen
import time 
import utils as u
import locale
import gettext

# Put the initial locale
cur_locale = locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain('tivion', cons.LOCALE_DIR)
gettext.textdomain('tivion')
_ = gettext.gettext

# Read the locale from config file
cfg = u.Config() 
cfg_lang = cfg.cfg.get('OPTIONS', 'Lang'.lower())
#print 'cfg_lang:', cfg_lang

try:
    import gtk
except:
    print _('You need install GTK to run Tivion.')
    sys.exit(1)
    
gtk_v = gtk.gtk_version
if gtk_v < (2, 14):
    print _('You need GTK 2.14 or higher to run Tivion.')
    sys.exit(1)

try:
    import pygtk

except:
    print _('You need install PyGTK to run Tivion.')
    sys.exit(1)

pygtk.require("2.0")
pygtk_v = gtk.pygtk_version
if pygtk_v < (2, 14):
    print _('You need PyGTK 2.14 or higher to run Tivion.')
    sys.exit(1)

### FIX for W32Codecs:
# Install this codecs if you have problems with windows channels
# wget http://www1.mplayerhq.hu/MPlayer/releases/codecs/essential-20071007.tar.bz2 && tar -xjf essential-*.tar.bz2 && sudo mkdir -p /usr/lib/win32 && sudo cp essential-*/* /usr/lib/win32

### FIX for ALSA:
# [AO OSS] audio_setup: Can't open audio device /dev/dsp: Device or resource busy
# -ao alsa
# Specify default audio driver (see 'mplayer -ao help' for a list).
# ao=alsa,oss,sdl,esd,arts

# [TODO] Cache should be customizable by interface.
# -cache 390 
# mplayer -input cmdlist
class Tivion():
    isPlaying = False
    isPaused  = False
    currentNameChannel = None
    volume = 50
    keepAbove = False
    winfullscreen = False
    last_msg_pushed = None
    ss_dir = os.environ['HOME'] + '/.tivion/'+ _('captures')

    def __init__(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(cons.VERSION)
        self.window.connect("destroy", self.on_destroy) # Needs testing: gtk.main_quit
        self.window.set_default_size(450, 320)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_gravity(gtk.gdk.GRAVITY_CENTER)
        self.window.set_border_width(0)
        try:
            gtk.window_set_default_icon_from_file(os.path.join(cons.IMG_PATH, 'tivion-icon.png'))
        except: # Give a try with develoment path
            gtk.window_set_default_icon_from_file(os.path.join(cons.DEV_IMG_PATH, 'tivion-icon.png'))
        self.window.connect("key-press-event", self.on_key_press_event)
        self.window.connect("delete_event", self.on_delete_event)
        self.window.connect("destroy_event", self.on_destroy_event)
        
        self.create_pipes()
        
        self.videoArea = gtk.DrawingArea();
        self.videoArea.set_size_request(320,280)
        
        vbox = gtk.VBox(0, True)
        vbox.set_border_width(0)
        self.menubar = self.__create_menubar()
        vbox.pack_start(self.menubar, False, False, 0)
        hbox = gtk.HBox(0,True)
        hbox.set_border_width(0)
        hbox.pack_start(self.videoArea, True, True, 0);
        
        self.sw = gtk.ScrolledWindow()
        self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.sw.set_size_request(200,-1)
        
        self.__Channels()
        import channels
        self.channels_set = channels.CHANNEL
        self.__insert_channels()

        self.sw.add_with_viewport(self.channels)
        
        self.vbox_panel = gtk.VBox(0, True)
        self.vbox_panel.set_border_width(0)

        self.vbox_panel.pack_start(self.__combobox_channels_type() , False, True, 0)
        self.vbox_panel.pack_start(self.sw, True, True, 0)
        self.vbox_panel.pack_start(self.__create_hbox_buttons(), False, False, 0)
        
        hbox.pack_start(self.vbox_panel, expand=False, fill=True, padding=0)
        vbox.pack_start(hbox, True, True, 0)
        self.statusbar = self.__create_statusbar()
        vbox.pack_end(self.statusbar, False, False, 0)
        self.window.add(vbox)
        self.window.show_all()
        
        win = self.videoArea.window
        w,h = win.get_size()
        color = gtk.gdk.Color(red=0, green=0, blue=0, pixel=0)      
        win.set_background(color)
        
        """ [FIXME] I need put the logo image on videoArea!
        gc = self.videoArea.get_style().black_gc
        pixbuf = gtk.gdk.pixbuf_new_from_file('tivion.png')
        self.videoArea.window.draw_pixbuf( gc, pixbuf, 0, 0, 32, 32,200, 200, gtk.gdk.RGB_DITHER_NORMAL, 0, 0) 
        """
        
        self.videoArea.show()

    def create_pipes(self):
        if os.path.exists(cons.FIFO):
            os.unlink(cons.FIFO)

        if os.path.exists(cons.STDOUT):
            os.unlink(cons.STDOUT)
        
        os.mkfifo(cons.FIFO)
    
    def __create_play_button(self):
        img_play_button = gtk.Image()
        img_play_button.set_from_icon_name('media-playback-start', gtk.ICON_SIZE_MENU) # gtk.STOCK_MEDIA_PLAY = missing?
        self.play_button = gtk.Button()
        self.play_button.set_image(img_play_button)
        self.play_button.connect('clicked', self.on_activate_pause)
    
    def __create_stop_button(self):
        img_stop_button = gtk.Image()
        img_stop_button.set_from_icon_name(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_MENU)
        self.stop_button = gtk.Button()
        self.stop_button.set_image(img_stop_button)
        self.stop_button.connect('clicked', self.on_activate_stop)

    def __create_streamVolumeButton(self):
        self.streamVolumeButton = gtk.VolumeButton()
        
        try:
            img_minus_button = gtk.Image()
            img_minus_button.set_from_icon_name(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
            minus_button = self.streamVolumeButton.get_minus_button()
            minus_button.set_label('-')
            minus_button.set_image(img_minus_button)
            
            img_plus_button = gtk.Image()
            img_plus_button.set_from_icon_name(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
            plus_button = self.streamVolumeButton.get_plus_button()
            plus_button.set_label('+')
            plus_button.set_image(img_plus_button)
        except AttributeError:
            print _('Error: fallback mode gtk.VolumeButton needs pygtk 2.16 for get_minus_button or get_plus_button')
        
        # [TODO] NOT WORKING! Have a bug with pygobject!
        #scale_button = self.streamVolumeButton.get_popup().get_child().get_child().get_children()
        #[<gtk.Button object at 0x8d28cd4 (GtkButton at 0x8dde760)>, <gtk.ScaleButtonScale object at 0x8d28824 (GtkScaleButtonScale at 0x8f0a060)>, <gtk.Button object at 0x8d28d4c (GtkButton at 0x8f06800)>]

        #print scale_button[1]
        """
        TypeError: Cannot create a consistent method resolution
        order (MRO) for bases ImplementorIface, Buildable, gtk.Orientable
        **
        ERROR:/build/buildd/pygobject-2.18.0/gobject/pygobject.c:924:pygobject_new_full: assertion failed: (tp != NULL)
        /usr/bin/tivion: line 5: 30042 Cancelado               python tivion.py $1 $2 $3 $4
        """
        
        #scale_button.set_digits(0)
        #scale_button.set_property("height-request", 160)  
        #scale_button.set_value_pos(gtk.POS_BOTTOM)
       
        #print self.streamVolumeButton.get_minus_button()

    def __create_hbox_buttons(self):
        self.__create_play_button()
        self.__create_stop_button()
        self.__create_streamVolumeButton()
        
        hbox_buttons = gtk.HBox(0,True)
        hbox_buttons.set_border_width(0)
        hbox_buttons.pack_start(self.play_button, True, True, 0);
        hbox_buttons.pack_start(self.stop_button, True, True, 0)
        """
        vol_button = gtk.Button()
        icon_theme = gtk.icon_theme_get_default()
        icon_vol_medium = icon_theme.load_icon('audio-volume-medium', gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG)
        img = gtk.Image()
        img.set_from_pixbuf(icon_vol_medium)
        #print dir(vol_button)
        vol_button.add(img)
        vol_button.connect('clicked', self.on_popup_volume)
        #hbox_buttons.pack_start(vol_button, True, True, 0); # [FIXME] Bug on popup
        """
        
        hbox_buttons.pack_start(self.streamVolumeButton , False, False, 0)
        return hbox_buttons
    
    def __combobox_channels_type(self):
        combobox = gtk.combo_box_new_text()
        combobox.append_text(_('All'))
        combobox.append_text(_('TV'))
        combobox.append_text(_('Radio'))
        combobox.append_text(_('Sopcast'))
        combobox.connect('changed', self.on_changed_combobox)
        combobox.set_active(0)
        return combobox
    
    def __create_menubar(self):
        setting = gtk.ImageMenuItem(_('Configuration'))
        setting.set_image(gtk.image_new_from_stock(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
        setting.connect('activate',self.on_activate_configuration)
        
        exit    = gtk.ImageMenuItem(_('Exit'))
        exit.set_image(gtk.image_new_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU))
        exit.connect('activate',self.on_destroy_event) # Maybe needs gtk.main_quit

        menu_archive = gtk.Menu()
        menu_archive.append(setting)
        menu_archive.append(exit)
        
        archive = gtk.MenuItem(_('File')) 
        archive.set_submenu(menu_archive)
        
        acel_pause = gtk.AccelGroup()
        self.window.add_accel_group(acel_pause)

        self.pause = gtk.Action(_('Pause/Resume'), _('_Pause/Resume'), _('_Pause/Resume'), 0)
        self.pause.create_icon(gtk.ICON_SIZE_MENU)
        try:
            self.pause.set_icon_name('media-playback-start') #Require gtk 2.16; gtk.STOCK_MEDIA_PLAY => Missing image
        except AttributeError:
            print _('Error: fallback mode gtk.Action needs pygtk 2.16 for set_icon_name: media-playback-start')

        self.pause.set_accel_group(acel_pause)
        self.pause.set_property('short-label', _('_Pause/Resume'))
        pausegroup = gtk.ActionGroup('BasicAction')
        pausegroup.add_action_with_accel(self.pause, '<Ctrl>space')
        self.pause.connect('activate',self.on_activate_pause)
        
        stop = gtk.ImageMenuItem(_('Stop'))
        stop.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_MENU))
        stop.connect('activate',self.on_activate_stop)
        
        self.volume_up = gtk.Action(_('Volume Up +10'), _('_Volume Up +10'), _('_Volume Up +10'), 0)
        self.volume_up.create_icon(gtk.ICON_SIZE_MENU)
        try:
            self.volume_up.set_icon_name('audio-volume-high')
        except AttributeError:
            print _('Error: fallback mode gtk.Action needs pygtk 2.16 for set_icon_name: audio-volume-high')
        self.volume_up.connect('activate',self.on_activate_volume_up)
        
        self.volume_down = gtk.Action(_('Volume Down -10'), _('_Volume Down -10'), _('_Volume Down -10'), 0)
        self.volume_down.create_icon(gtk.ICON_SIZE_MENU)
        try:
            self.volume_down.set_icon_name('audio-volume-low')
        except AttributeError:
            print _('Error: fallback mode gtk.Action needs pygtk 2.16 for set_icon_name: audio-volume-low')
            
        self.volume_down.connect('activate',self.on_activate_volume_down)
        
        ontop = gtk.ImageMenuItem(_('Window on top'))
        ontop.set_image(gtk.image_new_from_stock(gtk.STOCK_GOTO_TOP, gtk.ICON_SIZE_MENU))
        ontop.connect('activate',self.on_activate_ontop)
        
        ## show ALL ICONS!!
        #theme = gtk.icon_theme_get_default()
        #print theme.list_icons()
        
        """
        try:
            return gtk.gdk.pixbuf_new_from_file(get_app_logo(name))
        except:
            icontheme = gtk.icon_theme_get_default()
            icon = icontheme.lookup_icon('gtk-missing-image', 32, gtk.ICON_LOOKUP_NO_SVG)
            return icon.load_icon()
        """
        icon_theme = gtk.icon_theme_get_default()
        try:
            icon_camera = icon_theme.load_icon('camera-photo', gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG) # gtk.ICON_LOOKUP_USE_BUILTIN
        except gobject.GError, exc:
            print _("Can't load icon: "), exc


        acel_cap = gtk.AccelGroup()
        self.window.add_accel_group(acel_cap)
        #capture = gtk.ImageMenuItem('Take screenshoot')
        #capture.get_image().set_from_pixbuf(icon_camera)
        
        capture = gtk.Action(_('Take screenshot'), _('_Take screenshot'), _('Take screenshot'), 0)
        capture.create_icon(gtk.ICON_SIZE_MENU)
        try:
            capture.set_icon_name('camera-photo')
        except AttributeError:
            print _('Error: fallback mode gtk.Action needs pygtk 2.16 for set_icon_name: camera-photo')
            
        capture.set_property('short-label', _('_Take screenshot'))
        capturegroup = gtk.ActionGroup('BasicAction')# Create an ActionGroup named BasicAction
        capturegroup.add_action_with_accel(capture, '<Ctrl>S')
        capture.connect('activate',self.on_capture_screen)
        capture.set_accel_group(acel_cap)
        
        agr = gtk.AccelGroup()
        self.window.add_accel_group(agr)
        
        action = gtk.Action(_('Fullscreen'), _('_Fullscreen'), _('Fullscreen'), gtk.STOCK_FULLSCREEN)
        action.set_property('short-label', _('_Fullscreen'))
        action.connect('activate', self.on_activate_fullscreen) # Connect a callback to the action
    
        actiongroup = gtk.ActionGroup('BasicAction')# Create an ActionGroup named BasicAction
        actiongroup.add_action_with_accel(action, 'F11')
        action.set_accel_group(agr)# Have the action use accelgroup

        agrv = gtk.AccelGroup()
        self.viewChannels = gtk.CheckMenuItem(_('View channel list'))
        self.viewChannels.set_active(True)
        key, mod = gtk.accelerator_parse('F9')
        self.viewChannels.add_accelerator('activate', agrv, key, mod, gtk.ACCEL_VISIBLE)
        self.viewChannels.connect('activate', self.on_viewChannels)
        #self.viewChannels.connect("key_press_event", self.on_viewChannels)
        self.window.add_accel_group(agrv)

        menu_options = gtk.Menu()
        menu_options.append(ontop)
        menu_options.append(self.viewChannels)
        menu_options.append(action.create_menu_item())
        menu_options.append(capture.create_menu_item())
        
        options = gtk.MenuItem(_('Options'))
        options.set_submenu(menu_options)
        
        menu_sound = gtk.Menu()
        menu_sound.append(self.pause.create_menu_item())
        menu_sound.append(stop)
        menu_sound.append(self.volume_up.create_menu_item())
        menu_sound.append(self.volume_down.create_menu_item())
        sound = gtk.MenuItem(_('Sound'))
        sound.set_submenu(menu_sound)
        
        about = gtk.ImageMenuItem(_('About %s') % cons.VERSION)
        about.connect('activate', self.on_about)
        menu_help = gtk.Menu()
        menu_help.append(about)
        help = gtk.MenuItem(_('Help'))
        help.set_submenu(menu_help)

        menubar = gtk.MenuBar()
        menubar.append(archive)
        menubar.append(options)
        menubar.append(sound)
        menubar.append(help)

        return menubar  
        
    def __create_statusbar(self, initial = True):
        statusbar = gtk.Statusbar()
        if initial:
            context_id = statusbar.get_context_id('statusbar main')
            self.last_msg_pushed = _('%s loaded.') % cons.VERSION
            statusbar.push(context_id, self.last_msg_pushed)
        return statusbar 
        
    def push_status(self, msg):
        context_id = self.statusbar.get_context_id('statusbar main')
        self.last_msg_pushed = msg
        self.statusbar.push(context_id, msg)
        
    def pop_status(self):
        context_id = self.statusbar.get_context_id('statusbar main')
        self.statusbar.pop(context_id)
        
    def _execute(self, command, parseValue=False):
        """
        Executes a command, flush the buffer and 
        optionally returns the value from the key-value 
        raw value given by MPlayer
        """
        self.mplayerClient.write("%s\n" % command)
        self.mplayerClient.flush()
        line = self.progressLog.readline()
        answer = ""
        if line:
            if parseValue :
                if cons.DEBUG:
                    print _('Line: ') , line
                #answer = line[line.index('=')+1:-1]
            else: answer = line

        return answer 
    
    def stream_slider_changed(self,range,widget,new_value):
        #self.updateAppTooltip(self.pulseSink.client.name, new_value)
        #self.volumeAdjust.set_value(int(new_value)) 
        print int(new_value)
        return True

    
    def on_popup_volume(self, widget):
        volumeAdjust = gtk.Adjustment(self.volume, 0, 100, -5)
        streamVolume = gtk.VScale(volumeAdjust)
        streamVolume.set_digits(0)
        streamVolume.set_property("height-request", 160)  
        streamVolume.set_value_pos(gtk.POS_BOTTOM)
        streamVolume.connect("change-value",self.stream_slider_changed)
        
        
        muteButton = gtk.Button()
        muteButtonIcon = gtk.Image()
        muteButtonIcon.set_from_icon_name('audio-volume-muted', gtk.ICON_SIZE_MENU)
        muteButton.set_property("can-focus", False)
        muteButton.set_image(muteButtonIcon)
        muteButton.set_relief(gtk.RELIEF_NONE)

        add_button = gtk.Button()
        add_button = gtk.Image()
        add_button.set_from_icon_name(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
        
        remove_button = gtk.Button()
        remove_button = gtk.Image()
        remove_button.set_from_icon_name(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
        
        
        vbox = gtk.VBox(0, True)
        vbox.set_border_width(0)
        vbox.pack_start(muteButton, False, False, 0)
        vbox.pack_start(add_button, False, False, 0)
        vbox.pack_start(streamVolume, True, True, 0)
        vbox.pack_start(remove_button, False, False, 0)
        
        
        self.window_popup = gtk.Window(gtk.WINDOW_POPUP)
        self.window_popup.set_position(gtk.WIN_POS_MOUSE)
        self.window_popup.add(vbox)

        self.window_popup.connect("scroll-event", self.stream_scroll_event)
        self.window_popup.show_all()

    def stream_scroll_event(self, widget, event): 
        ##print "entro"
        """volume = self.volumeAdjust.get_value()

        if event.direction == gtk.gdk.SCROLL_UP or \
           event.direction == gtk.gdk.SCROLL_RIGHT:
          volume += 10
        elif event.direction == gtk.gdk.SCROLL_DOWN or \
             event.direction == gtk.gdk.SCROLL_LEFT:
          volume -= 10

        if volume > 100:
          volume = 100
        elif volume < 0:
          volume = 0

        #self.updateAppTooltip(self.pulseSink.client.name, volume) 
        self.volumeAdjust.set_value(volume)
        """
        return True
    
    def on_about(self, widget):
        about = gtk.AboutDialog()
        about.set_transient_for(self.window)
        about.set_program_name("Tivion")
        about.set_version(cons.VERSION_NUMBER)
        about.set_copyright("Copyright (C) 2009 Ángel Guzmán Maeso (Shakaran) <shakaran@gmail.com>")
        about.set_authors(['Ángel Guzmán Maeso (Shakaran) <shakaran@gmail.com>'])
        try:
            about.set_license(open(cons.license_filename).read())
        except:
            about.set_license(_("GNU General Public License version 3 or later.\nSee http://www.gnu.org/licenses/gpl-3.0.txt"))
        about.set_comments(_('A single player and recorder for streaming online television and radio channels composed of multicultural and international level.'))
        about.set_website('http://www.shakaran.es')
        about.set_name('Tivion')
        try:
            about.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(cons.IMG_PATH, 'tivion.png')))
        except: # Give a try with develoment path
             about.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(cons.DEV_IMG_PATH, 'tivion.png')))       
        """
        TRANSLASTORS: Maintain the names of translators here.
        Launchpad does this automatically for translations
        typed in Launchpad. This is a special string shown 
        in the 'About' box.
        """
        about.set_translator_credits(_('translator-credits'))
        about.run()
        about.destroy()
        
    def on_changed_combobox(self, combobox):
        model = combobox.get_model()
        index = combobox.get_active()

        if index != None:
            if model[index][0] == 'TV':
                self.__insert_channels(cons.TV)
            elif model[index][0] == 'Radio':
                self.__insert_channels(cons.RADIO)
            elif model[index][0] == 'Sopcast':
                self.__insert_channels(cons.SOPCAST)
            elif model[index][0] == 'All':
                self.__insert_channels(None)
                
        return

    def get_lang(self, index):
        for (lang, ulang, name) in self.found_lang:
            if name == index:
                return (lang, ulang, name)

    def on_changed_language(self, combobox):
        global _, gettext, cur_locale
        destroywin = False
        model = combobox.get_model()
        index = combobox.get_active()

        (lang, ulang, name) = self.get_lang(model[index][0])
        os.environ['LANG'] = ulang # Dont work with os.putenv('LANG', ulang)
        cur_locale = locale.setlocale(locale.LC_ALL, '')
        t = gettext.translation('tivion', languages=[lang])
        t.install()
        _ = t.lgettext

        md = gtk.MessageDialog(self.wconfig, 
            gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_QUESTION, 
            0, _('Do you want reload the application for apply language changes?'))
        #md.set_default_size(200, 200)
        md.set_modal(True)
        md.set_title(_('Apply changes'))
        self.window.set_sensitive(False)
        self.wconfig.set_sensitive(False)
        md.set_position(gtk.WIN_POS_CENTER)
        md.add_button(gtk.STOCK_YES, gtk.RESPONSE_OK)
        md.add_button(gtk.STOCK_NO, gtk.RESPONSE_CANCEL)
        md.set_default_response(gtk.RESPONSE_CANCEL)
        
        response = md.run()

        if response == gtk.RESPONSE_OK:
            vbox = self.window.get_children()[0]
            vbox.get_children()[0].destroy() # Destroy menubar
            self.menubar = self.__create_menubar()
            vbox.pack_start(self.menubar, False, False, 0)
            vbox.reorder_child(self.menubar, 0)
            
            """ [TODO] Update the right channel zone
            hbox = vbox.get_children()[1]
            hbox_buttons = self.vbox_panel.get_children()[2] # Buttons
            self.vbox_panel.destroy() # Destroy combobox, treestore and buttons
            
            self.vbox_panel = gtk.VBox(0, True)
            self.vbox_panel.set_border_width(0)
            self.vbox_panel.pack_start(self.__combobox_channels_type() , False, True, 0)
            #self.vbox_panel.pack_start(self.sw, True, True, 0)
            self.vbox_panel.pack_startself.vbox_panel.pack_start(hbox_buttons, False, False, 0)
            hbox.pack_end(self.vbox_panel, expand=False, fill=True, padding=0)
            #hbox.reorder_child(self.vbox_panel, 1)
            
            #print subhbox.get_children()[1].get_children()[0].get_children()
            #subhbox.get_children()[0].destroy() # Combobox channels
            #subhbox.get_children()[1].get_children()[0].destroy() # Viewport
            #subhbox.get_children()[1].destroy() # Scrolled window
            """
        
            vbox.get_children()[2].destroy() # Destroy statusbar
            self.statusbar = self.__create_statusbar(False)
            self.push_status( _('%s loaded.') % cons.VERSION)
            vbox.pack_end(self.statusbar, False, False, 0)
            vbox.reorder_child(self.statusbar, 2)
            
            destroywin = True
            
            self.menubar.show()
            self.window.show_all()
        
        self.window.set_sensitive(True)
        self.wconfig.set_sensitive(True)
        md.destroy()
        
        if destroywin:
            self.wconfig.hide()

        return

    def on_key_press_event(self, widget, event) :
        if gtk.gdk.keyval_name(event.keyval) == 'F9':
            self.on_viewChannels(self.viewChannels)
        elif event.string == 'q' :
            self.on_destroy()
        
        if cons.DEBUG:
            keyname = gtk.gdk.keyval_name(event.keyval)
            print _('Key pressed: '), keyname
    
    def on_viewChannels(self, widget):
        if widget.active: 
            self.vbox_panel.show()
            #widget.set_active(False)
        else:
            self.vbox_panel.hide()
            #widget.set_active(True)

    def on_wconfig_destroy(self, widget, other = None):
        widget.destroy()
        self.window.set_sensitive(True)
    
    def on_activate_configuration(self, widget):
        self.wconfig = gtk.Window()
        self.wconfig.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.wconfig.set_transient_for(self.window)
        self.wconfig.set_title(_('Configuration'))
        self.wconfig.set_default_size(250, 320)
        self.wconfig.set_border_width(5)
        self.wconfig.connect("delete_event", self.on_wconfig_destroy)
        self.wconfig.connect("destroy_event", self.on_wconfig_destroy)
        self.wconfig.connect("destroy", self.on_wconfig_destroy)
        self.window.set_sensitive(False)
        
        vbox = gtk.VBox(0, True)
        vbox.set_border_width(0)
        
        (user_locale, user_locale_encode) = locale.getdefaultlocale()
        
        cur_locale = '%s-%s' % (user_locale, user_locale_encode)
        
        label = gtk.Label(_('Language: '))
        ###label.set_line_wrap(True) #break lines
        
        self.supported_languages = [
                                ('en',    'en_US.UTF-8', _('English')), 
                                ('en_GB', 'en_GB.UTF-8', _('English (United Kingdom)')), 
                                ('fr',    'fr_RF.UTF-8', _('French')), 
                                ('es',    'es_ES.UTF-8', _('Spanish')), 
                                ('ca',    'ca_ES.UTF-8', _('Catalan')), 
                                ('it',    'it_IT.UTF-8', _('Italian')), 
                                ('pt',    'pt_PT.UTF-8', _('Portuguese')), 
                                ('ru',    'ru_TU.UTF-8', _('Russian')), 
                                ('tr',    'tr_TR.UTF-8', _('Turkish'))
                              ]
        # Other option: gettext.find('tivion', cons.LOCALE_DIR, ['es', 'en'], 'All')
        self.found_lang = []
        for (lang, ulang, name) in self.supported_languages:
            path = '/usr/share/locale/%s/LC_MESSAGES/tivion.po' % lang
            if os.path.exists(path) and os.path.isfile(path):
                if cons.DEBUG:
                    print _('Found language: \'%s\'' %lang) + ('(%s) in' %name) + ('%s' %path)
                self.found_lang.append((lang, ulang, name))

        combobox = gtk.combo_box_new_text()
        if not self.found_lang:
            combobox.append_text(_('English'))
            combobox.set_active(0)
        else:
            i = 0
            for (lang, ulang, name) in self.found_lang:
                combobox.append_text(name)
                if ulang == os.getenv('LANG'):
                    combobox.set_active(i)
                i += 1

        combobox.connect('changed', self.on_changed_language)
        
        hbox = gtk.HBox(0, True)
        hbox.set_border_width(0)
        
        hbox.pack_start(label, False, False, 0)
        hbox.pack_start(combobox, False, False, 0)
        
        hbox_ss = gtk.HBox(0, True)
        hbox_ss.set_border_width(0)
        
        ss_btn = gtk.FileChooserButton(_('Select a folder'))
        ss_btn.set_current_folder(self.get_ss_folder())
        ss_btn.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        ss_btn.set_size_request(120, -1)
        ss_btn.set_show_hidden(True)
        ss_btn.connect('current-folder-changed', self.on_ss_selected) 

        hbox_ss.pack_start(gtk.Label('Screenshot folder:'), False, False, 0)
        hbox_ss.pack_start(ss_btn, False, False, 0)

        vbox.pack_start(hbox, False, False, 0)
        vbox.pack_start(hbox_ss, False, False, 0)

        self.wconfig.add(vbox)
        self.wconfig.show_all()
    
    def check_ss_dir(self):
        u.check_dir(self.ss_dir)
    
    def get_ss_folder(self):
        ss_folder = cfg.cfg.get('OPTIONS', 'Screenshot_folder')
        if not ss_folder:
            ss_folder = self.ss_dir
        return ss_folder

    def on_ss_selected(self, widget):
        filename = widget.get_filename()
        if cons.DEBUG:
            print _('Screenshot folder:'), filename    
        cfg.cfg.set('OPTIONS', 'Screenshot_folder', filename) 
        cfg.write()

    def on_activate_stop(self, widget):
        if self.isPlaying and not self.isStop: # [FIXME] The mplayer windows is frozen here.
            self._execute('stop')
            self.isStop = True
            self.push_status(_('Stopped: %s') % self.currentNameChannel)
        else:
             self.push_status(_('Nothing playing!'))
    
    def on_activate_fullscreen(self, widget):
        if self.winfullscreen == False:
            #window1.set_decorated(gtk.FALSE)
            self.window.fullscreen()
            # X11 need this, OpenGL not
            #self.videoArea.set_size_request(-1,-1)
            #self._execute('vo_fullscreen')
        else:
            self.window.unfullscreen()
        self.winfullscreen = not self.winfullscreen
    
    def on_activate_ontop(self, widget): #  Window.present() and Window.set_keep_above()
        if self.isPlaying and not self.isStop: # [FIXME] Dont working!, debe ser por pygtk que usa la window
            self.keepAbove = not self.keepAbove
            self.window.set_keep_above(self.keepAbove)
            self._execute('vo_ontop')
        else:
             self.push_status(_('Nothing playing!'))
    
    def on_activate_volume_up(self, widget): # [FIXME] Hacer un deslizable + teclas arriba y abajo
        if self.isPlaying and not self.isStop:
            self.volume = self.volume + 10
            if self.volume > 100: 
                self.volume = 100;
            self._execute('volume %s 1' % self.volume)
        else:
             self.push_status(_('Nothing playing!'))
    
    def on_activate_volume_down(self, widget):
        if self.isPlaying and not self.isStop:
            self.volume = self.volume - 10
            if self.volume < 0: 
                self.volume = 0;
            self._execute('volume %s 1' % self.volume)
        else:
             self.push_status(_('Nothing playing!'))
             
    def on_capture_screen(self, widget): # [FIXME] Hacer el menos y tambien un deslizable + teclas    
        if self.isPlaying:
            import gst
            time.sleep(.3) # For avoid capture the menu and mix frames (slow graphic cards)
            self.pipeline = gst.Pipeline("mypipeline")

            #def snap_shot(self,widget,data=None):
            #global file_loc   ,pixbuf
            self.pipeline.set_state(gst.STATE_PAUSED)
            alloc = self.videoArea.get_allocation()
            pixbuf = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, False, 8, alloc.width, alloc.height)
            pixbuf.get_from_drawable(self.videoArea.window, self.videoArea.get_colormap(), 0, 0, 0, 0, alloc.width, alloc.height)
            
            self.ss_dir = self.get_ss_folder()
            self.check_ss_dir()
            file_loc = self.ss_dir  + '/tivion-' + _('capture') + '-%d.jpg' % time.time()       
            pixbuf.save(file_loc,'jpeg', {'quality':'100'})
            print _('Screenshot file stored in: %s') % file_loc
            self.pipeline.set_state(gst.STATE_PLAYING)
        else:
             self.push_status(_('Tivion could not make a capture. Nothing playing!'))
    
    def on_activate_pause(self, widget):
        if self.isPlaying and not self.isStop:
            self.mplayerClient = open(cons.FIFO, 'w')
            self.mplayerClient.write('pause\n') 
            self.mplayerClient.flush()  
            self.pop_status()
            if self.isPaused == True:
                if self.currentNameChannel != None:
                    self.push_status(_('Playing: %s') % self.currentNameChannel)
                    self.pause.set_icon_name('media-playback-start') #gtk.STOCK_MEDIA_PLAY => Missing image
                    img_play_button = gtk.Image()
                    img_play_button.set_from_icon_name('media-playback-start', gtk.ICON_SIZE_MENU) # 'media-playback-start'
                    self.play_button.set_image(img_play_button)
                else:
                    self.push_status(_('Nothing to play!'))
                self.isPaused = False
            elif self.isPaused == False:
                if self.currentNameChannel != None:
                    self.push_status(_('Paused: %s') % self.currentNameChannel)
                    self.pause.set_icon_name(gtk.STOCK_MEDIA_PAUSE) 
                    img_play_button = gtk.Image()
                    img_play_button.set_from_icon_name(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU) # 'media-playback-start'
                    self.play_button.set_image(img_play_button)
                else:
                    self.push_status(_('Nothing to pause!'))
                self.isPaused = True
        else:
             self.push_status(_('Nothing playing!'))
            
    
    
    def on_destroy(self, widget = None):
        if hasattr(self,'mplayerClient'):
            try:
                self.mplayerClient.write('quit\n')
                self.mplayerClient.flush()    
                self.mplayerClient.close()
            except ValueError: # [FIXME] This need a fix: #ValueError: I/O operation on closed file
                pass
        
        if hasattr(self,'proc'):
            self.proc.terminate()      
                
        if os.path.exists(cons.FIFO):
            os.unlink(cons.FIFO)

        if os.path.exists(cons.STDOUT):
            os.unlink(cons.STDOUT)

        gtk.main_quit()
        return False
    
    def on_delete_event(self, widget, data = None):
        return self.on_destroy()

    def on_destroy_event(self, widget, data = None):
        return self.on_destroy()
        
    def selectChannel(self, widget, event):
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            treeselection = widget.get_selection()
            (model, iter) = treeselection.get_selected()
            if iter != None: # Avoid double click on expand tree
                id = model.get_value(iter, cons.COLUMN_ID)
                if id == 0:
                    path = model.get_path(iter)
                    if self.channels.row_expanded(path):
                        self.channels.collapse_row(path)
                    else:
                        self.channels.expand_row(path, True)
                elif id != 0:
                    name = model.get_value(iter, cons.COLUMN_NAME)
                    country = model.get_value(iter, cons.COLUMN_COUNTRY)
                    url= model.get_value(iter, cons.COLUMN_URL)
                    print "ID:", id, country, name, url
                    self.currentNameChannel = name
                    
                    if url[0:6] == 'FILMON':
                        if cons.DEBUG:
                            print _('Parsing FILMON url: %s' % url[6:8])

                        url = self.parseFilmon(url[6:8])

                    if url != None:
                        if self.isPlaying == True:
                            #try:
                            self._execute("loadfile '%s' 0\n" % url,False) 
                            self.mplayerClient.flush() 
                            #except IOerror: # [FIXME] flush() falling if dont have internet connection
                            #    print 'IOerror'
                            
                            #self.proc.terminate()
                            
                            #if os.path.exists(FIFO):
                            #    os.unlink(FIFO)

                            #if os.path.exists(STDOUT):
                            #    os.unlink(STDOUT)
                        else:
                            ## [TODO] SOPCAST
                            #  /usr/bin/sp-auth/sp-sc-auth SOPCAST-CHANNEL:PORT/???? ???? LOCAL-PORT
                            #  /usr/bin/sp-auth/sp-sc-auth sop://sop.rojadirecta.com:3912/6002 3908 8908
                            # Mplayer needs a good cache for this (8 MB recommended)
                            # mplayer -prefer-ipv4 -nolirc -ao alsa -vo gl2 -cache 8192 -framedrop http://localhost:8908

                            
                            # [TODO] Ipv4 o ipv6 should be customizable
                            # -framedrop for old ATI cards (like mine! ATI Mobility Radeon X600(r380))
                            # If on fullscreen is slow you can change -vo option to other option (complete list: $mplayer -vo help)
                            command = 'mplayer -prefer-ipv4 -slave -idle -nolirc -nomouseinput -vo %s -framedrop -ao alsa -cache 390 -wid %i %s -input file=%s' % (cons.DEFAULT_VO, self.videoArea.window.xid, url, cons.FIFO) 
                            if cons.DEBUG: 
                                print 'DEFAULT_VO:',cons.DEFAULT_VO
                                print _('Note: You can change DEFAULT_VO with other options that can be found with the command "mplayer -vo help"')
                                print command
                            commandList = command.split()
                            self.proc = Popen(commandList, stdout=open(cons.STDOUT,"w+b"), stderr=open(cons.STDOUT,"r+b"))
                            
                            self.isPlaying = True
                            self.isPaused  = False
                            self.isStop    = False
                            cons.lastProcess = self.proc
                            
                            self.mplayerClient = open(cons.FIFO,"w")
                            self.progressLog = open(cons.STDOUT,"r")

                            self._execute("play", False)  
                            
                            # [TODO] Show osd msg
                            #msg = "hello"
                            #self._execute('osd_show_text "%s"\n' % msg, False) 
                            
                        self.push_status(_('Playing: %s') % name)
    
    def __Channels(self):
        self.channels = gtk.TreeView()
        self.channels.set_headers_clickable(True)
        self.channels.set_headers_visible(True)
        self.channels.set_rules_hint(True)
        self.channels.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.channels.connect('button_press_event', self.selectChannel)
        self.__create_model()
        self.channels.expand_all()
       
    def __create_model(self):
        self.__set_model()
        self.model.clear()
        self.__add_columns()

    def __set_model(self):
        import gobject
        self.model = gtk.TreeStore(
                    gobject.TYPE_INT,       # ID
                    gobject.TYPE_STRING,    # COUNTRY
                    gobject.TYPE_INT,       # TYPE
                    gobject.TYPE_STRING,    # NAME
                    gobject.TYPE_STRING)    # URL
        self.channels.set_model(self.model)
        
    def __add_columns(self):
        column = gtk.TreeViewColumn(_('Channels'))
        renderer = gtk.CellRendererText()
        column.pack_start(renderer, True)
        column.set_attributes(renderer, markup = cons.COLUMN_NAME)
        column.set_resizable(False)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_expand(False)
        column.set_sort_column_id(cons.COLUMN_COUNTRY)
        self.channels.append_column(column)
    
    def __insert_channels(self, type = None):
        self.model.clear()
        countries = {}
        
        for c in self.channels_set:
            if countries.get(unicode(c[1])) == None:
                iter = self.model.append(None, [0, '', -1, c[1], '']) # Only put the country on name place (Channel 0 reserved)
                countries[unicode(c[1])] = iter
            else:
                iter = countries.get(unicode(c[1]))
            
            if type == None or type == 'All' or type == c[2]:
                self.insert_data(c[0], c[1], c[2], c[3], c[4], iter)
        
        # Remove empty country rows
        for row in self.model:
            if not self.model.iter_has_child(row.iter):
                self.model.remove(row.iter)
    
    def insert_data(self, id, country, type, name, url, iter):
        self.model.append(iter, [int(id), unicode(country), int(type), unicode(name), unicode(url)])
    
    def __readXML(self, url):
        import urllib
        
        params = urllib.urlencode({'u': 'hdi_p', 'p': 'Uio43-Iud345-', 'a': 'hdi_api_get_channel_info', 'id': int(url)})
        f = urllib.urlopen("http://www.filmon.com/tv/?%s" % params)
        return f.read()

    def __parseXML(self, xmldata):
        """ Trying with XML """
        import xml.dom.minidom
        from xml.dom.minidom import Node
    
        doc = xml.dom.minidom.parseString(xmldata)
        try:
            url = doc.getElementsByTagName("url")[0].childNodes[0].data
            print url
        except IndexError:
            url = None
        
        if url != None:
            try:
                title = doc.getElementsByTagName("title")[0].childNodes[0].data
                print title
            except IndexError:
                title = None 
        
        return (url, title)

    def __parseXMLRegex(self, xmldata, url):
        """ Trying with Regex """
        if url == None:
            import re
            p = re.compile(r'(<\!\[CDATA\[)(.*)(]]>)')
            m = p.search(xmldata)
            if m:
                url = m.group()[9:-3]
                if cons.DEBUG: 
                    print _('Match found: %s' % url)
            else:
                url = None
                if cons.DEBUG: 
                    print _('No match')
        return url

    def parseFilmon(self, url):
        xmldata = self.__readXML(url)
        
        if xmldata != None and xmldata != 'ERROR: channel not found!':
            if cons.DEBUG:
                print _('XMLDATA:')
                print xmldata
                
            (url, title) = self.__parseXML(xmldata)
            url = self.__parseXMLRegex(xmldata, url)
            if url == None:
                print _('Error: Channel Filmon parsing failed! Please report.')
        return url

if __name__ == '__main__':
    try:
        if len(sys.argv) > 1:
            for i in range(len(sys.argv)):
                if sys.argv[i] in ['-d','-debug']:
                    cons.DEBUG = True

                if sys.argv[i] in ['-c','-channel-list']:
                    from channels import CHANNEL_VERSION, CHANNEL
                    from time import strftime, gmtime
                    print _('Channel list (%s):') %strftime("%A, %d %B %Y %H:%M:%S", gmtime(int(CHANNEL_VERSION)))
                    
                    # Thanks to the IRC channel #pyar for his magic!
                    import operator
                    d = {}
                    for id, country, type, name, url in CHANNEL:
                        d.setdefault(country, []).append((name, type))
                    
                    ordered = sorted(d.items(), key = lambda x: len(x[1]), reverse = True)
                    p , nn = 0, 0
                    for country, nc in ordered:
                        p = p + 1
                        print _('Country %s - ' %p) + _('%s - Total: ' %country) + ('%d' %len(nc))
                        for c, type in sorted(nc):
                            nn = nn + 1
                            if type == 0: 
                                type = _('TV')
                            elif type == 1: 
                                type = _('Radio')
                            elif type == 2: 
                                type = _('Sopcast') 
                            print '  %s - %s (%s)' % (nn, c, type)
                        print 

                    print 'Total countries: %s\nTotal channels: %s' %(p, nn)
                    sys.exit(0)
                    
                if sys.argv[i] == '-vo':
                    if i+1 in range(len(sys.argv)):
                      if sys.argv[i+1] != None and sys.argv[i+1] in ['xv','x11','xover','gl','gl2','sdl']: 
                        cons.DEFAULT_VO = sys.argv[i+1] # [FIXME] SDL display another window!
                      else:
                        cons.DEFAULT_VO = 'gl2'     
                
                if sys.argv[i] in ['-h','-help']:
                    print cons.VERSION
                    print _('Help: todo')
                    sys.exit(0)
                
        if cons.DEBUG:
            print _('*** DEBUG ***')
            print _('You can see additional output logs of Mplayer in %s') % cons.STDOUT
            print _('Note: the output log is removed when the program is closed.')
            print 'GTK: %s.%s.%s' %(gtk_v[0], gtk_v[1], gtk_v[2])
            print 'PyGTK: %s.%s.%s' %(pygtk_v[0], pygtk_v[1], pygtk_v[2])
            print _('Username:'), os.environ.get('USERNAME')
            print _('User language:'), os.environ['LANG']
            print _('Current locale:'), cur_locale
            print _('Current domain:'), gettext._current_domain
            print _('Default locale dir:'), gettext._default_localedir
            print _('Gettext package:'), gettext.__package__

        if u.check_player():
            app = Tivion()
            gtk.main()
        else:
            print _("MPlayer must be installed and be found in /usr/bin") #and Serpentine, VLC: /usr/bin/vlc
    except KeyboardInterrupt:
        """
        [FIXME]
        if hasattr(self,"mplayerClient"):
            self.mplayerClient.write("quit\n")
            self.mplayerClient.flush()    """
                
        if os.path.exists(cons.FIFO):
            os.unlink(cons.FIFO)

        if os.path.exists(cons.STDOUT):
            os.unlink(cons.STDOUT)
           
        if cons.lastProcess != None:
            cons.lastProcess.terminate()
        
        print _("Program terminated by keyboard.")
        sys.exit(1)
